@@ -1,27 +1,13 @@
 /*  chat.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef _GNU_SOURCE
-#define _GNU_SOURCE    /* needed for wcswidth() */
+#define _GNU_SOURCE    /* needed for strcasestr() and wcswidth() */
 #endif
 
 #include "chat.h"
@@ -65,7 +51,7 @@ static void kill_infobox(ToxWindow *self);
 #endif /* AUDIO */
 
 /* Array of chat command names used for tab completion. */
-static const char *chat_cmd_list[] = {
+static const char *const chat_cmd_list[] = {
     "/accept",
     "/add",
     "/autoaccept",
@@ -131,7 +117,7 @@ static const char *chat_cmd_list[] = {
 
 static void set_self_typingstatus(ToxWindow *self, Toxic *toxic, bool is_typing)
 {
-    if (toxic->c_config->show_typing_self == SHOW_TYPING_OFF) {
+    if (!toxic->c_config->show_typing_self) {
         return;
     }
 
@@ -158,54 +144,59 @@ void kill_chat_window(ToxWindow *self, Toxic *toxic)
 #endif /* AUDIO */
 
     kill_all_file_transfers_friend(toxic, self->num);
-    log_disable(ctx->log);
-    line_info_cleanup(ctx->hst);
-    cqueue_cleanup(ctx->cqueue);
 
-    delwin(ctx->linewin);
-    delwin(ctx->history);
+    if (ctx != NULL) {
+        log_disable(ctx->log);
+        line_info_cleanup(ctx->hst);
+        cqueue_cleanup(ctx->cqueue);
+
+        delwin(ctx->linewin);
+        delwin(ctx->history);
+
+        free(ctx->log);
+        free(ctx);
+    }
+
     delwin(statusbar->topline);
-
-    free(ctx->log);
-    free(ctx);
     free(self->help);
     free(statusbar);
 
-    disable_chatwin(self->num);
+    disable_friend_window(self->num);
     kill_notifs(self->active_box);
-    del_window(self, toxic->c_config);
+    del_window(self, toxic->windows, toxic->c_config);
 }
 
-static void recv_message_helper(ToxWindow *self, const Client_Config *c_config, const char *msg,
+static void recv_message_helper(ToxWindow *self, const Toxic *toxic, const char *msg,
                                 const char *nick)
 {
+    const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
     line_info_add(self, c_config, true, nick, NULL, IN_MSG, 0, 0, "%s", msg);
-    write_to_log(ctx->log, c_config, msg, nick, false);
+    write_to_log(ctx->log, c_config, msg, nick, LOG_HINT_NORMAL_I);
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
+        box_notify2(self, toxic, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
                     self->active_box, "%s", msg);
     } else {
-        box_notify(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
+        box_notify(self, toxic, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
                    &self->active_box, nick, "%s", msg);
     }
 }
 
-static void recv_action_helper(ToxWindow *self, const Client_Config *c_config, const char *action,
-                               const char *nick)
+static void recv_action_helper(ToxWindow *self, const Toxic *toxic,  const char *action, const char *nick)
 {
+    const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
     line_info_add(self, c_config, true, nick, NULL, IN_ACTION, 0, 0, "%s", action);
-    write_to_log(ctx->log, c_config, action, nick, true);
+    write_to_log(ctx->log, c_config, action, nick, LOG_HINT_ACTION);
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
+        box_notify2(self, toxic, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
                     self->active_box, "* %s %s", nick, action);
     } else {
-        box_notify(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
+        box_notify(self, toxic, generic_message, NT_WNDALERT_1 | NT_NOFOCUS | c_config->bell_on_message,
                    &self->active_box, self->name, "* %s %s", nick, action);
     }
 }
@@ -219,22 +210,20 @@ static void chat_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Mess
         return;
     }
 
-    const Client_Config *c_config = toxic->c_config;
-
     if (self->num != num) {
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(toxic->tox, nick, num);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(name, sizeof(name), num);
 
     if (type == TOX_MESSAGE_TYPE_NORMAL) {
-        recv_message_helper(self, c_config, msg, nick);
+        recv_message_helper(self, toxic, msg, name);
         return;
     }
 
     if (type == TOX_MESSAGE_TYPE_ACTION) {
-        recv_action_helper(self, c_config, msg, nick);
+        recv_action_helper(self, toxic, msg, name);
         return;
     }
 }
@@ -248,7 +237,6 @@ static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num,
         return;
     }
 
-    Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
     if (self->num != num) {
@@ -259,23 +247,21 @@ static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num,
     ChatContext *ctx = self->chatwin;
     const char *msg;
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, num);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(name, sizeof(name), num);
 
     Tox_Connection prev_status = statusbar->connection;
     statusbar->connection = connection_status;
 
-    if (c_config->show_connection_msg == SHOW_WELCOME_MSG_OFF) {
-        return;
-    }
-
     if (prev_status == TOX_CONNECTION_NONE) {
         chat_resume_file_senders(self, toxic, num);
         file_send_queue_check(self, toxic, self->num);
 
-        msg = "has come online";
-        line_info_add(self, c_config, true, nick, NULL, CONNECTION, 0, GREEN, msg);
-        write_to_log(ctx->log, c_config, msg, nick, true);
+        if (c_config->show_connection_msg) {
+            msg = "has come online";
+            line_info_add(self, c_config, true, name, NULL, CONNECTION, 0, GREEN, "%s", msg);
+            write_to_log(ctx->log, c_config, msg, name, LOG_HINT_CONNECT);
+        }
     } else if (connection_status == TOX_CONNECTION_NONE) {
         Friends.list[num].is_typing = false;
 
@@ -285,9 +271,11 @@ static void chat_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num,
 
         chat_pause_file_transfers(num);
 
-        msg = "has gone offline";
-        line_info_add(self, c_config, true, nick, NULL, DISCONNECTION, 0, RED, msg);
-        write_to_log(ctx->log, c_config, msg, nick, true);
+        if (c_config->show_connection_msg) {
+            msg = "has gone offline";
+            line_info_add(self, c_config, true, name, NULL, DISCONNECTION, 0, RED, "%s", msg);
+            write_to_log(ctx->log, c_config, msg, name, LOG_HINT_DISCONNECT);
+        }
     }
 }
 
@@ -327,6 +315,36 @@ static void chat_onNickChange(ToxWindow *self, Toxic *toxic, uint32_t num, const
     set_window_title(self, statusbar->nick, length);
 }
 
+static void chat_onNickRefresh(ToxWindow *self, Toxic *toxic)
+{
+    if (self == NULL || toxic == NULL) {
+        return;
+    }
+
+    StatusBar *statusbar = self->stb;
+
+    char new_name[TOXIC_MAX_NAME_LENGTH + 1];
+    const uint16_t n_len = get_friend_name(new_name, sizeof(new_name), self->num);
+
+    if (strcmp(new_name, statusbar->nick) == 0) {
+        return;
+    }
+
+    char self_key[TOX_ADDRESS_SIZE];
+    tox_self_get_address(toxic->tox, (uint8_t *) self_key);
+
+    char other_key[TOX_PUBLIC_KEY_SIZE];
+
+    if (get_friend_public_key(other_key, self->num)) {
+        if (rename_logfile(toxic->windows, toxic->c_config, statusbar->nick, new_name, self_key, other_key,
+                           self->id) != 0) {
+            fprintf(stderr, "failed to rename logfile\n");
+        }
+    }
+
+    chat_onNickChange(self, toxic, self->num, new_name, n_len);
+}
+
 static void chat_onStatusChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_User_Status status)
 {
     UNUSED_VAR(toxic);
@@ -572,8 +590,7 @@ static void chat_onFileControl(ToxWindow *self, Toxic *toxic, uint32_t friendnum
                 ft->state = FILE_TRANSFER_STARTED;
                 line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "File transfer [%zu] for '%s' accepted.",
                               ft->index, ft->file_name);
-                sound_notify(self, c_config, silent,
-                             NT_NOFOCUS | c_config->bell_on_filetrans_accept | NT_WNDALERT_2, NULL);
+                sound_notify(self, toxic, silent, NT_NOFOCUS | c_config->bell_on_filetrans_accept | NT_WNDALERT_2, NULL);
             } else if (ft->state == FILE_TRANSFER_PAUSED) {    /* transfer is resumed */
                 ft->state = FILE_TRANSFER_STARTED;
             }
@@ -785,10 +802,10 @@ static void chat_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t friendnum, u
     free(file_path);
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, transfer_pending, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_filetrans,
+        box_notify2(self, toxic, transfer_pending, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_filetrans,
                     self->active_box, "Incoming file: %s", filename);
     } else {
-        box_notify(self, c_config, transfer_pending, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_filetrans,
+        box_notify(self, toxic, transfer_pending, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_filetrans,
                    &self->active_box, self->name, "Incoming file: %s", filename);
     }
 
@@ -837,17 +854,16 @@ static void chat_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t frien
     Friends.list[friendnumber].conference_invite.length = length;
     Friends.list[friendnumber].conference_invite.type = type;
 
-    char name[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(toxic->tox, name, friendnumber);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(name, sizeof(name), friendnumber);
 
     const char *description = type == TOX_CONFERENCE_TYPE_AV ? "an audio conference" : "a conference";
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, self->active_box,
+        box_notify2(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, self->active_box,
                     "invites you to join %s", description);
     } else {
-        box_notify(self, c_config, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, &self->active_box,
-                   name,
+        box_notify(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, &self->active_box, name,
                    "invites you to join %s", description);
     }
 
@@ -884,16 +900,16 @@ static void chat_onGroupInvite(ToxWindow *self, Toxic *toxic, uint32_t friendnum
     memcpy(Friends.list[friendnumber].group_invite.data, invite_data, length);
     Friends.list[friendnumber].group_invite.length = length;
 
-    sound_notify(self, c_config, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, NULL);
+    sound_notify(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, NULL);
 
-    char name[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(toxic->tox, name, friendnumber);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(name, sizeof(name), friendnumber);
 
     if (self->active_box != -1) {
-        box_silent_notify2(self, c_config, NT_WNDALERT_2 | NT_NOFOCUS, self->active_box,
+        box_silent_notify2(self, toxic, NT_WNDALERT_2 | NT_NOFOCUS, self->active_box,
                            "invites you to join group chat");
     } else {
-        box_silent_notify(self, c_config, NT_WNDALERT_2 | NT_NOFOCUS, &self->active_box, name,
+        box_silent_notify(self, toxic, NT_WNDALERT_2 | NT_NOFOCUS, &self->active_box, name,
                           "invites you to join group chat");
     }
 
@@ -963,14 +979,14 @@ static void chat_onGameInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_num
     Friends.list[friend_number].game_invite.pending = true;
     Friends.list[friend_number].game_invite.data_length = data_length;
 
-    char name[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(toxic->tox, name, friend_number);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    get_friend_name(name, sizeof(name), friend_number);
 
     if (self->active_box != -1) {
-        box_notify2(self, c_config, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, self->active_box,
+        box_notify2(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, self->active_box,
                     "invites you to play %s", game_string);
     } else {
-        box_notify(self, c_config, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, &self->active_box,
+        box_notify(self, toxic, generic_message, NT_WNDALERT_2 | c_config->bell_on_invite, &self->active_box,
                    name,
                    "invites you to play %s", game_string);
     }
@@ -1007,14 +1023,13 @@ static void chat_onInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_number,
                   "Incoming audio call! Type: \"/answer\" or \"/reject\"");
 
     if (self->ringing_sound == -1) {
-        sound_notify(self, c_config, call_incoming, NT_LOOP | c_config->bell_on_invite, &self->ringing_sound);
+        sound_notify(self, toxic, call_incoming, NT_LOOP | c_config->bell_on_invite, &self->ringing_sound);
     }
 
     if (self->active_box != -1) {
-        box_silent_notify2(self, c_config, NT_NOFOCUS | NT_WNDALERT_0, self->active_box, "Incoming audio call!");
+        box_silent_notify2(self, toxic, NT_NOFOCUS | NT_WNDALERT_0, self->active_box, "Incoming audio call!");
     } else {
-        box_silent_notify(self, c_config, NT_NOFOCUS | NT_WNDALERT_0, &self->active_box, self->name,
-                          "Incoming audio call!");
+        box_silent_notify(self, toxic, NT_NOFOCUS | NT_WNDALERT_0, &self->active_box, self->name, "Incoming audio call!");
     }
 }
 
@@ -1038,7 +1053,7 @@ static void chat_onRinging(ToxWindow *self, Toxic *toxic, uint32_t friend_number
 #ifdef SOUND_NOTIFY
 
     if (self->ringing_sound == -1) {
-        sound_notify(self, c_config, call_outgoing, NT_LOOP, &self->ringing_sound);
+        sound_notify(self, toxic, call_outgoing, NT_LOOP, &self->ringing_sound);
     }
 
 #endif /* SOUND_NOTIFY */
@@ -1281,7 +1296,7 @@ static void send_action(ToxWindow *self, ChatContext *ctx, Toxic *toxic, char *a
         return;
     }
 
-    char selfname[TOX_MAX_NAME_LENGTH];
+    char selfname[TOX_MAX_NAME_LENGTH + 1];
     tox_self_get_name(toxic->tox, (uint8_t *) selfname);
 
     const size_t len = tox_self_get_name_size(toxic->tox);
@@ -1306,7 +1321,10 @@ static bool chat_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
     ChatContext *ctx = self->chatwin;
     StatusBar *statusbar = self->stb;
 
-    int x, y, y2, x2;
+    int x;
+    int y;
+    int y2;
+    int x2;
     getyx(self->window, y, x);
     getmaxyx(self->window, y2, x2);
 
@@ -1326,7 +1344,7 @@ static bool chat_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
     }
 
     if (ltr || key == L'\n') {    /* char is printable */
-        input_new_char(self, c_config, key, x, x2);
+        input_new_char(self, toxic, key, x, x2);
 
         if (ctx->line[0] != '/' && !ctx->self_is_typing && statusbar->connection != TOX_CONNECTION_NONE) {
             set_self_typingstatus(self, toxic, true);
@@ -1339,11 +1357,11 @@ static bool chat_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
         return true;
     }
 
-    bool input_ret = input_handle(self, c_config, key, x, x2);
+    bool input_ret = input_handle(self, toxic, key, x, x2);
 
     if (key == L'\t' && ctx->len > 1 && ctx->line[0] == '/') {    /* TAB key: auto-complete */
         input_ret = true;
-        int diff = -1;
+        int diff;
 
         /* TODO: make this not suck */
         if (wcsncmp(ctx->line, L"/sendfile ", wcslen(L"/sendfile ")) == 0) {
@@ -1363,13 +1381,13 @@ static bool chat_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
             diff = complete_line(self, toxic, chat_cmd_list, sizeof(chat_cmd_list) / sizeof(char *));
         }
 
-        if (diff != -1) {
+        if (diff >= 0) {
             if (x + diff > x2 - 1) {
-                int wlen = MAX(0, wcswidth(ctx->line, sizeof(ctx->line) / sizeof(wchar_t)));
+                const int wlen = MAX(0, wcswidth(ctx->line, sizeof(ctx->line) / sizeof(wchar_t)));
                 ctx->start = wlen < x2 ? 0 : wlen - x2 + 1;
             }
         } else {
-            sound_notify(self, c_config, notif_error, 0, NULL);
+            sound_notify(self, toxic, notif_error, 0, NULL);
         }
 
     } else if (key == L'\r') {
@@ -1397,13 +1415,13 @@ static bool chat_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
                     execute(ctx->history, self, toxic, line, CHAT_COMMAND_MODE);
                 }
             } else if (line[0]) {
-                char selfname[TOX_MAX_NAME_LENGTH];
+                char selfname[TOX_MAX_NAME_LENGTH + 1];
                 tox_self_get_name(tox, (uint8_t *) selfname);
 
-                size_t len = tox_self_get_name_size(tox);
+                const size_t len = tox_self_get_name_size(tox);
                 selfname[len] = '\0';
 
-                int id = line_info_add(self, c_config, true, selfname, NULL, OUT_MSG, 0, 0, "%s", line);
+                const int id = line_info_add(self, c_config, true, selfname, NULL, OUT_MSG, 0, 0, "%s", line);
                 cqueue_add(ctx->cqueue, line, strlen(line), OUT_MSG, id);
             } else {
                 line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
@@ -1546,7 +1564,7 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
         tox_friend_get_status_message(toxic->tox, self->num, (uint8_t *) statusmsg, NULL);
         const size_t s_len = tox_friend_get_status_message_size(toxic->tox, self->num, NULL);
 
-        filter_str(statusmsg, s_len);
+        filter_string(statusmsg, s_len, false);
         snprintf(statusbar->statusmsg, sizeof(statusbar->statusmsg), "%s", statusmsg);
         statusbar->statusmsg_len = strlen(statusbar->statusmsg);
 
@@ -1556,15 +1574,15 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
     self->x = x2;
 
     /* Truncate note if it doesn't fit in statusbar */
-    size_t maxlen = x2 - getcurx(statusbar->topline) - KEY_IDENT_BYTES - 6;
+    const int maxlen = x2 - getcurx(statusbar->topline) - KEY_IDENT_BYTES - 6;
 
     pthread_mutex_lock(&Winthread.lock);
-    size_t statusmsg_len = statusbar->statusmsg_len;
+    const size_t statusmsg_len = statusbar->statusmsg_len;
     pthread_mutex_unlock(&Winthread.lock);
 
-    if (statusmsg_len > maxlen) {
+    if (statusmsg_len > maxlen && maxlen >= 3) {
         pthread_mutex_lock(&Winthread.lock);
-        statusbar->statusmsg[maxlen - 3] = 0;
+        statusbar->statusmsg[maxlen - 3] = '\0';
         strcat(statusbar->statusmsg, "...");
         statusbar->statusmsg_len = maxlen;
         pthread_mutex_unlock(&Winthread.lock);
@@ -1614,10 +1632,10 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
 
     UNUSED_VAR(x);
 
-    int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
+    const int new_x = ctx->start ? x2 - 1 : MAX(0, wcswidth(ctx->line, ctx->pos));
     wmove(self->window, y, new_x);
 
-    draw_window_bar(self);
+    draw_window_bar(self, toxic->windows);
 
     wnoutrefresh(self->window);
 
@@ -1644,11 +1662,13 @@ static void chat_onDraw(ToxWindow *self, Toxic *toxic)
 
 static void chat_init_log(ToxWindow *self, Toxic *toxic, const char *self_nick)
 {
+    Tox *tox = toxic->tox;
+
     ChatContext *ctx = self->chatwin;
     const Client_Config *c_config = toxic->c_config;
 
     char myid[TOX_ADDRESS_SIZE];
-    tox_self_get_address(toxic->tox, (uint8_t *) myid);
+    tox_self_get_address(tox, (uint8_t *) myid);
 
     if (log_init(ctx->log, c_config, self_nick, myid, Friends.list[self->num].pub_key, LOG_TYPE_CHAT) != 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to initialize chat log.");
@@ -1681,7 +1701,7 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
-        exit_toxic_err("failed in chat_onInit", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in chat_onInit");
     }
 
     self->x = x2;
@@ -1692,20 +1712,22 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     statusbar->status = get_friend_status(self->num);
     statusbar->connection = get_friend_connection_status(self->num);
 
-    char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH];
-    tox_friend_get_status_message(tox, self->num, (uint8_t *) statusmsg, NULL);
+    const size_t s_len = tox_friend_get_status_message_size(tox, self->num, NULL);
 
-    size_t s_len = tox_friend_get_status_message_size(tox, self->num, NULL);
-    statusmsg[s_len] = '\0';
+    if (s_len > 0 && s_len <= TOX_MAX_STATUS_MESSAGE_LENGTH) {
+        char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH + 1] = {0};
+        tox_friend_get_status_message(tox, self->num, (uint8_t *) statusmsg, NULL);
+        statusmsg[s_len] = '\0';
+        filter_string(statusmsg, s_len, false);
+        snprintf(statusbar->statusmsg, sizeof(statusbar->statusmsg), "%s", statusmsg);
+    }
 
-    filter_str(statusmsg, s_len);
-    snprintf(statusbar->statusmsg, sizeof(statusbar->statusmsg), "%s", statusmsg);
     statusbar->statusmsg_len = strlen(statusbar->statusmsg);
 
-    char nick[TOX_MAX_NAME_LENGTH + 1];
-    size_t n_len = get_nick_truncate(tox, nick, self->num);
-    memcpy(statusbar->nick, nick, n_len);
-    statusbar->nick[n_len] = 0;
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    const uint16_t n_len = get_friend_name(name, sizeof(name), self->num);
+
+    snprintf(statusbar->nick, sizeof(statusbar->nick), "%s", name);
     statusbar->nick_len = n_len;
 
     /* Init subwindows */
@@ -1721,7 +1743,7 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     ctx->cqueue = calloc(1, sizeof(struct chat_queue));
 
     if (ctx->log == NULL || ctx->hst == NULL || ctx->cqueue == NULL) {
-        exit_toxic_err("failed in chat_onInit", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in chat_onInit");
     }
 
     line_info_init(ctx->hst);
@@ -1732,7 +1754,7 @@ static void chat_onInit(ToxWindow *self, Toxic *toxic)
     friend_set_logging_enabled(self->num, friend_config_get_autolog(self->num));
     friend_set_auto_file_accept(self->num, friend_config_get_auto_accept_files(self->num));
 
-    chat_init_log(self, toxic, nick);
+    chat_init_log(self, toxic, name);
 
     execute(ctx->history, self, toxic, "/log", GLOBAL_COMMAND_MODE);  // Print log status to screen
 
@@ -1747,7 +1769,7 @@ ToxWindow *new_chat(Tox *tox, uint32_t friendnum)
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
 
     if (ret == NULL) {
-        exit_toxic_err("failed in new_chat", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_chat");
     }
 
     ret->type = WINDOW_TYPE_CHAT;
@@ -1755,6 +1777,7 @@ ToxWindow *new_chat(Tox *tox, uint32_t friendnum)
     ret->onKey = &chat_onKey;
     ret->onDraw = &chat_onDraw;
     ret->onInit = &chat_onInit;
+    ret->onNickRefresh = &chat_onNickRefresh;
     ret->onMessage = &chat_onMessage;
     ret->onConnectionChange = &chat_onConnectionChange;
     ret->onTypingChange = & chat_onTypingChange;
@@ -1789,16 +1812,17 @@ ToxWindow *new_chat(Tox *tox, uint32_t friendnum)
 
     ret->active_box = -1;
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    size_t n_len = get_nick_truncate(tox, nick, friendnum);
-    set_window_title(ret, nick, n_len);
+    char name[TOXIC_MAX_NAME_LENGTH + 1];
+    const uint16_t n_len = get_friend_name(name, sizeof(name), friendnum);
+
+    set_window_title(ret, name, n_len);
 
     ChatContext *chatwin = calloc(1, sizeof(ChatContext));
     StatusBar *stb = calloc(1, sizeof(StatusBar));
     Help *help = calloc(1, sizeof(Help));
 
     if (stb == NULL || chatwin == NULL || help == NULL) {
-        exit_toxic_err("failed in new_chat", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_chat");
     }
 
     ret->chatwin = chatwin;
