@@ -1,23 +1,9 @@
 /*  chat_commands.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include "chat_commands.h"
@@ -58,7 +44,7 @@ void cmd_autoaccept_files(WINDOW *window, ToxWindow *self, Toxic *toxic, int arg
             msg = "Auto-file accept for this friend is disabled; type \"/autoaccept on\" to enable";
         }
 
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", msg);
         return;
     }
 
@@ -74,7 +60,7 @@ void cmd_autoaccept_files(WINDOW *window, ToxWindow *self, Toxic *toxic, int arg
         msg = "Invalid option. Use \"/autoaccept on\" and \"/autoaccept off\" to toggle auto-file accept";
     }
 
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", msg);
 }
 
 void cmd_cancelfile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
@@ -94,7 +80,7 @@ void cmd_cancelfile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, cha
 
     char msg[MAX_STR_SIZE];
     const char *inoutstr = argv[1];
-    long int idx = strtol(argv[2], NULL, 10);
+    const long int idx = strtol(argv[2], NULL, 10);
 
     if ((idx == 0 && strcmp(argv[2], "0")) || idx >= MAX_FILES || idx < 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid file ID.");
@@ -149,7 +135,7 @@ void cmd_conference_invite(WINDOW *window, ToxWindow *self, Toxic *toxic, int ar
         return;
     }
 
-    long int conferencenum = strtol(argv[1], NULL, 10);
+    const long int conferencenum = strtol(argv[1], NULL, 10);
 
     if ((conferencenum == 0 && strcmp(argv[1], "0")) || conferencenum < 0 || conferencenum == LONG_MAX) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid conference number.");
@@ -181,14 +167,9 @@ void cmd_conference_join(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     const char *conferencekey = Friends.list[self->num].conference_invite.key;
-    uint16_t length = Friends.list[self->num].conference_invite.length;
-    uint8_t type = Friends.list[self->num].conference_invite.type;
+    const uint16_t length = Friends.list[self->num].conference_invite.length;
+    const uint8_t type = Friends.list[self->num].conference_invite.type;
 
     if (!Friends.list[self->num].conference_invite.pending) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "No pending conference invite.");
@@ -254,11 +235,6 @@ void cmd_group_accept(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, c
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     if (Friends.list[self->num].group_invite.length == 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "No pending group invite");
         return;
@@ -277,15 +253,15 @@ void cmd_group_accept(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, c
         return;
     }
 
-    size_t nick_len = tox_self_get_name_size(tox);
+    const size_t nick_len = tox_self_get_name_size(tox);
     char self_nick[TOX_MAX_NAME_LENGTH + 1];
     tox_self_get_name(tox, (uint8_t *) self_nick);
     self_nick[nick_len] = '\0';
 
     Tox_Err_Group_Invite_Accept err;
-    uint32_t groupnumber = tox_group_invite_accept(tox, self->num, Friends.list[self->num].group_invite.data,
-                           Friends.list[self->num].group_invite.length, (const uint8_t *) self_nick, nick_len,
-                           (const uint8_t *) passwd, passwd_len, &err);
+    const uint32_t groupnumber = tox_group_invite_accept(tox, self->num, Friends.list[self->num].group_invite.data,
+                                 Friends.list[self->num].group_invite.length, (const uint8_t *) self_nick, nick_len,
+                                 (const uint8_t *) passwd, passwd_len, &err);
 
     if (err != TOX_ERR_GROUP_INVITE_ACCEPT_OK) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to join group (error %d).", err);
@@ -313,7 +289,7 @@ void cmd_group_invite(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, c
         return;
     }
 
-    int groupnumber = atoi(argv[1]);
+    const int groupnumber = atoi(argv[1]);
 
     if (groupnumber == 0 && strcmp(argv[1], "0")) {    /* atoi returns 0 value on invalid input */
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid group number.");
@@ -348,15 +324,10 @@ void cmd_game_join(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         return;
     }
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     GameType type = Friends.list[self->num].game_invite.type;
-    uint32_t id = Friends.list[self->num].game_invite.id;
+    const uint32_t id = Friends.list[self->num].game_invite.id;
     uint8_t *data = Friends.list[self->num].game_invite.data;
-    size_t length = Friends.list[self->num].game_invite.data_length;
+    const size_t length = Friends.list[self->num].game_invite.data_length;
 
     const int ret = game_initialize(self, toxic, type, id, data, length, false);
 
@@ -404,7 +375,7 @@ void cmd_savefile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         return;
     }
 
-    long int idx = strtol(argv[1], NULL, 10);
+    const long int idx = strtol(argv[1], NULL, 10);
 
     if ((idx == 0 && strcmp(argv[1], "0")) || idx < 0 || idx >= MAX_FILES) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "No pending file transfers with ID %ld", idx);
@@ -495,7 +466,7 @@ void cmd_sendfile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
 
     char path[MAX_STR_SIZE];
     snprintf(path, sizeof(path), "%s", argv[1]);
-    int path_len = strlen(path);
+    const int path_len = strlen(path);
 
     if (path_len >= MAX_STR_SIZE) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "File path exceeds character limit.");
@@ -509,7 +480,7 @@ void cmd_sendfile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         return;
     }
 
-    off_t filesize = file_size(path);
+    const off_t filesize = file_size(path);
 
     if (filesize <= 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid file.");
@@ -518,11 +489,17 @@ void cmd_sendfile(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
     }
 
     char file_name[TOX_MAX_FILENAME_LENGTH];
-    size_t namelen = get_file_name(file_name, sizeof(file_name), path);
+    const int namelen = get_file_name(file_name, sizeof(file_name), path);
+
+    if (namelen < 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to fetch file name (OOM)");
+        fclose(file_to_send);
+        return;
+    }
 
     Tox_Err_File_Send err;
-    uint32_t filenum = tox_file_send(tox, self->num, TOX_FILE_KIND_DATA, (uint64_t) filesize, NULL,
-                                     (uint8_t *) file_name, namelen, &err);
+    const uint32_t filenum = tox_file_send(tox, self->num, TOX_FILE_KIND_DATA, (uint64_t) filesize, NULL,
+                                           (uint8_t *) file_name, namelen, &err);
 
     if (err != TOX_ERR_FILE_SEND_OK) {
         goto on_send_error;
@@ -558,7 +535,7 @@ on_send_error:
         }
 
         case TOX_ERR_FILE_SEND_FRIEND_NOT_CONNECTED: {
-            int queue_idx = file_send_queue_add(self->num, path, path_len);
+            const int queue_idx = file_send_queue_add(self->num, path, path_len);
 
             char msg[MAX_STR_SIZE];
 
