@@ -1,23 +1,9 @@
 /*  friendlist.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include <arpa/inet.h>
@@ -66,7 +52,8 @@ static struct PendingDel {
 
 typedef enum Default_Conf {
     Default_Conf_Auto_Accept_Files = 0,
-    Default_Conf_Tab_Name_Colour = WHITE_BAR_FG,
+    Default_Conf_Tab_Name_Colour = BAR_TEXT,
+    Default_Conf_Alias_Set = 0,
 } Default_Conf;
 
 static void set_default_friend_config_settings(ToxicFriend *friend, const Client_Config *c_config)
@@ -78,9 +65,23 @@ static void set_default_friend_config_settings(ToxicFriend *friend, const Client
     Friend_Settings *settings = &friend->settings;
 
     settings->auto_accept_files = Default_Conf_Auto_Accept_Files != 0;
-    settings->autolog = c_config->autolog == AUTOLOG_ON;
-    settings->show_connection_msg = c_config->show_connection_msg == SHOW_CONNECTION_MSG_ON;
+    settings->autolog = c_config->autolog;
+    settings->show_connection_msg = c_config->show_connection_msg;
     settings->tab_name_colour = Default_Conf_Tab_Name_Colour;
+    settings->alias_set = Default_Conf_Alias_Set != 0;
+}
+
+void friend_reset_default_config_settings(const Client_Config *c_config)
+{
+    for (size_t i = 0; i < Friends.max_idx; ++i) {
+        ToxicFriend *friend = &Friends.list[i];
+
+        if (!friend->active) {
+            continue;
+        }
+
+        set_default_friend_config_settings(friend, c_config);
+    }
 }
 
 static void realloc_friends(int n)
@@ -97,7 +98,7 @@ static void realloc_friends(int n)
     uint32_t *f_idx = realloc(Friends.index, n * sizeof(uint32_t));
 
     if (f == NULL || f_idx == NULL) {
-        exit_toxic_err("failed in realloc_friends", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in realloc_friends");
     }
 
     Friends.list = f;
@@ -118,14 +119,14 @@ static void realloc_blocklist(int n)
     uint32_t *b_idx = realloc(Blocked.index, n * sizeof(uint32_t));
 
     if (b == NULL || b_idx == NULL) {
-        exit_toxic_err("failed in realloc_blocklist", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in realloc_blocklist");
     }
 
     Blocked.list = b;
     Blocked.index = b_idx;
 }
 
-void kill_friendlist(ToxWindow *self, const Client_Config *c_config)
+void kill_friendlist(ToxWindow *self, Windows *windows, const Client_Config *c_config)
 {
     for (size_t i = 0; i < Friends.max_idx; ++i) {
         if (Friends.list[i].active) {
@@ -143,7 +144,7 @@ void kill_friendlist(ToxWindow *self, const Client_Config *c_config)
     realloc_blocklist(0);
     realloc_friends(0);
     free(self->help);
-    del_window(self, c_config);
+    del_window(self, windows, c_config);
 }
 
 static void clear_blocklist_index(size_t idx)
@@ -393,8 +394,8 @@ static void update_friend_last_online(uint32_t num, time_t timestamp, const char
     Friends.list[num].last_online.tm = *localtime((const time_t *)&timestamp);
 
     /* if the format changes make sure TIME_STR_SIZE is the correct size */
-    strftime(Friends.list[num].last_online.hour_min_str, TIME_STR_SIZE, timestamp_format,
-             &Friends.list[num].last_online.tm);
+    format_time_str(Friends.list[num].last_online.hour_min_str, TIME_STR_SIZE, timestamp_format,
+                    &Friends.list[num].last_online.tm);
 }
 
 static void friendlist_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Message_Type type, const char *str,
@@ -409,30 +410,23 @@ static void friendlist_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, To
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (num >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[num].chatwin != -1) {
+    if (Friends.list[num].window_id != -1) {
         return;
     }
 
-    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
+    const int window_id = add_window(toxic, new_chat(tox, Friends.list[num].num));
+
+    if (window_id < 0) {
+        fprintf(stderr, "Failed to create new chat window in friendlist_onMessage\n");
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, num);
-
-    line_info_add(home_window, c_config, true, nick, NULL, IN_MSG, 0, 0, "%s", str);
-    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
-                  "* Warning: Too many windows are open.");
-
-    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+    Friends.list[num].window_id = window_id;
 }
 
 static void friendlist_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Connection connection_status)
@@ -487,8 +481,8 @@ static void friendlist_onNickChange(ToxWindow *self, Toxic *toxic, uint32_t num,
     tox_self_get_address(toxic->tox, (uint8_t *) myid);
 
     if (strcmp(oldname, newnamecpy) != 0) {
-        if (rename_logfile(toxic->c_config, oldname, newnamecpy, myid, Friends.list[num].pub_key,
-                           Friends.list[num].chatwin) != 0) {
+        if (rename_logfile(toxic->windows, toxic->c_config, oldname, newnamecpy, myid, Friends.list[num].pub_key,
+                           Friends.list[num].window_id) != 0) {
             fprintf(stderr, "Failed to rename friend chat log from `%s` to `%s`\n", oldname, newnamecpy);
         }
     }
@@ -496,6 +490,14 @@ static void friendlist_onNickChange(ToxWindow *self, Toxic *toxic, uint32_t num,
     sort_friendlist_index();
 }
 
+static void friendlist_onNickRefresh(ToxWindow *self, Toxic *toxic)
+{
+    UNUSED_VAR(self);
+    UNUSED_VAR(toxic);
+
+    sort_friendlist_index();
+}
+
 static void friendlist_onStatusChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_User_Status status)
 {
     UNUSED_VAR(self);
@@ -544,7 +546,7 @@ void friendlist_onFriendAdded(ToxWindow *self, Toxic *toxic, uint32_t num, bool
 
         Friends.list[i].num = num;
         Friends.list[i].active = true;
-        Friends.list[i].chatwin = -1;
+        Friends.list[i].window_id = -1;
         Friends.list[i].auto_accept_files = false;  // do not change
         Friends.list[i].connection_status = TOX_CONNECTION_NONE;
         Friends.list[i].status = TOX_USER_STATUS_NONE;
@@ -566,10 +568,10 @@ void friendlist_onFriendAdded(ToxWindow *self, Toxic *toxic, uint32_t num, bool
 
         update_friend_last_online(i, t, c_config->timestamp_format);
 
-        char tempname[TOX_MAX_NAME_LENGTH + 1];
-        int name_len = get_nick_truncate(tox, tempname, num);
-        memcpy(Friends.list[i].name, tempname, name_len);
-        Friends.list[i].name[name_len] = 0;
+        char tempname[TOXIC_MAX_NAME_LENGTH + 1];
+        const size_t name_len = get_nick_truncate(tox, tempname, sizeof(tempname), num);
+
+        snprintf(Friends.list[i].name, sizeof(Friends.list[i].name), "%s", tempname);
         Friends.list[i].namelength = name_len;
 
         if (i == Friends.max_idx) {
@@ -581,7 +583,11 @@ void friendlist_onFriendAdded(ToxWindow *self, Toxic *toxic, uint32_t num, bool
         }
 
 #ifdef AUDIO
-        init_friend_AV(i);
+
+        if (!init_friend_AV(i)) {
+            fprintf(stderr, "Failed to init AV for friend %u\n", i);
+        }
+
 #endif
 
         return;
@@ -603,7 +609,7 @@ static void friendlist_add_blocked(const Client_Config *c_config, uint32_t fnum,
 
         Friends.list[i].num = fnum;
         Friends.list[i].active = true;
-        Friends.list[i].chatwin = -1;
+        Friends.list[i].window_id = -1;
         Friends.list[i].status = TOX_USER_STATUS_NONE;
         Friends.list[i].namelength = Blocked.list[bnum].namelength;
         update_friend_last_online(i, Blocked.list[bnum].last_on, c_config->timestamp_format);
@@ -619,7 +625,11 @@ static void friendlist_add_blocked(const Client_Config *c_config, uint32_t fnum,
         sort_friendlist_index();
 
 #ifdef AUDIO
-        init_friend_AV(i);
+
+        if (!init_friend_AV(i)) {
+            fprintf(stderr, "Failed to init AV for friend %d\n", i);
+        }
+
 #endif
         return;
     }
@@ -639,29 +649,23 @@ static void friendlist_onGameInvite(ToxWindow *self, Toxic *toxic, uint32_t frie
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (friend_number >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[friend_number].chatwin != -1) {
+    if (Friends.list[friend_number].window_id != -1) {
         return;
     }
 
-    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[friend_number].chatwin = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
+    const int window_id = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
+
+    if (window_id < 0) {
+        fprintf(stderr, "Failed to create new chat window in friendlist_onGameInvite\n");
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, friend_number);
-
-    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
-                  "* Game invite from %s failed: Too many windows are open.", nick);
-
-    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+    Friends.list[friend_number].window_id = window_id;
 }
 
 #endif // GAMES
@@ -675,31 +679,23 @@ static void friendlist_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t num, u
     UNUSED_VAR(name_length);
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (num >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[num].chatwin != -1) {
+    if (Friends.list[num].window_id != -1) {
         return;
     }
 
-    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
+    const int window_id = add_window(toxic, new_chat(tox, Friends.list[num].num));
+
+    if (window_id < 0) {
+        fprintf(stderr, "Failed to create new chat window in friendlist_onFileRecv\n");
         return;
     }
 
-    tox_file_control(tox, num, filenum, TOX_FILE_CONTROL_CANCEL, NULL);
-
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, num);
-
-    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
-                  "* File transfer from %s failed: too many windows are open.", nick);
-
-    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+    Friends.list[num].window_id = window_id;
 }
 
 static void friendlist_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t num, uint8_t type,
@@ -716,29 +712,23 @@ static void friendlist_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (num >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[num].chatwin != -1) {
+    if (Friends.list[num].window_id != -1) {
         return;
     }
 
-    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
+    const int window_id = add_window(toxic, new_chat(tox, Friends.list[num].num));
+
+    if (window_id < 0) {
+        fprintf(stderr, "Failed to create new chat window in friendlist_onConferenceInvite\n");
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, num);
-
-    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
-                  "* Conference chat invite from %s failed: too many windows are open.", nick);
-
-    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+    Friends.list[num].window_id = window_id;
 }
 
 static void friendlist_onGroupInvite(ToxWindow *self, Toxic *toxic, uint32_t num, const char *data, size_t length,
@@ -753,29 +743,23 @@ static void friendlist_onGroupInvite(ToxWindow *self, Toxic *toxic, uint32_t num
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (num >= Friends.max_idx) {
         return;
     }
 
-    if (Friends.list[num].chatwin != -1) {
+    if (Friends.list[num].window_id != -1) {
         return;
     }
 
-    if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
+    const int window_id = add_window(toxic, new_chat(tox, Friends.list[num].num));
+
+    if (window_id < 0) {
+        fprintf(stderr, "Failed to create new chat window in friendlist_onGroupInvite\n");
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(tox, nick, num);
-
-    line_info_add(home_window, c_config, NULL, NULL, NULL, SYS_MSG, 0, RED,
-                  "* Group chat invite from %s failed: too many windows are open.", nick);
-
-    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+    Friends.list[num].window_id = window_id;
 }
 
 /* move friendlist/blocklist cursor up and down */
@@ -818,13 +802,13 @@ static void delete_friend(Toxic *toxic, uint32_t f_num)
         --Friends.num_online;
     }
 
-    /* close friend's chatwindow if it's currently open */
-    if (Friends.list[f_num].chatwin >= 0) {
-        ToxWindow *toxwin = get_window_ptr(Friends.list[f_num].chatwin);
+    /* close friend's window if it's currently open */
+    if (Friends.list[f_num].window_id >= 0) {
+        ToxWindow *toxwin = get_window_pointer_by_id(toxic->windows, Friends.list[f_num].window_id);
 
         if (toxwin != NULL) {
             kill_chat_window(toxwin, toxic);
-            set_active_window_index(1);   /* keep friendlist focused */
+            set_active_window_by_type(toxic->windows, WINDOW_TYPE_FRIEND_LIST);
         }
     }
 
@@ -1012,8 +996,6 @@ static bool friendlist_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
     if (self->help->active) {
         help_onKey(self, key);
@@ -1061,17 +1043,18 @@ static bool friendlist_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr
             }
 
             /* Jump to chat window if already open */
-            if (Friends.list[f].chatwin != -1) {
-                set_active_window_index(Friends.list[f].chatwin);
-            } else if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-                Friends.list[f].chatwin = add_window(toxic, new_chat(tox, Friends.list[f].num));
-                set_active_window_index(Friends.list[f].chatwin);
-            } else {
-                const char *msg = "* Warning: Too many windows are open.";
-                line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED, msg);
-                sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+            if (Friends.list[f].window_id < 0) {
+                const int window_id = add_window(toxic, new_chat(tox, Friends.list[f].num));
+
+                if (window_id < 0) {
+                    fprintf(stderr, "Failed to create new chat window in friendlist_onKey\n");
+                    return true;
+                }
+
+                Friends.list[f].window_id = window_id;
             }
 
+            set_active_window_by_id(toxic->windows, Friends.list[f].window_id);
             break;
 
         case KEY_DC:
@@ -1127,9 +1110,7 @@ static void blocklist_onDraw(ToxWindow *self, Tox *tox, int y2, int x2)
     int start = (y2 - FLIST_OFST) * page;
     int end = y2 - FLIST_OFST + start;
 
-    int i;
-
-    for (i = start; i < Blocked.num_blocked && i < end; ++i) {
+    for (int i = start; i < Blocked.num_blocked && i < end; ++i) {
         uint32_t f = Blocked.index[i];
         bool f_selected = false;
 
@@ -1170,9 +1151,7 @@ static void blocklist_onDraw(ToxWindow *self, Tox *tox, int y2, int x2)
         wprintw(self->window, "Public key: ");
         wattroff(self->window, A_BOLD);
 
-        int i;
-
-        for (i = 0; i < TOX_PUBLIC_KEY_SIZE; ++i) {
+        for (int i = 0; i < TOX_PUBLIC_KEY_SIZE; ++i) {
             wprintw(self->window, "%02X", Blocked.list[selected_num].pub_key[i] & 0xff);
         }
     }
@@ -1197,7 +1176,7 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
     int x2, y2;
     getmaxyx(self->window, y2, x2);
 
-    bool fix_statuses = x2 != self->x;    /* true if window max x value has changed */
+    const bool fix_statuses = x2 != self->x;    /* true if window max x value has changed */
 
     wattron(self->window, COLOR_PAIR(CYAN));
     wprintw(self->window, " Press the");
@@ -1207,14 +1186,14 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
     wprintw(self->window, "key for help\n\n");
     wattroff(self->window, COLOR_PAIR(CYAN));
 
-    draw_window_bar(self);
+    draw_window_bar(self, toxic->windows);
 
     if (blocklist_view == 1) {
         blocklist_onDraw(self, toxic->tox, y2, x2);
         return;
     }
 
-    time_t cur_time = get_unix_time();
+    const time_t cur_time = get_unix_time();
     struct tm cur_loc_tm = *localtime((const time_t *) &cur_time);
 
     wattron(self->window, A_BOLD);
@@ -1241,9 +1220,7 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
     const size_t num_friends = Friends.num_friends;
     pthread_mutex_unlock(&Winthread.lock);
 
-    int i;
-
-    for (i = start; i < num_friends && i < end; ++i) {
+    for (int i = start; i < num_friends && i < end; ++i) {
         pthread_mutex_lock(&Winthread.lock);
         uint32_t f = Friends.index[i];
         bool is_active = Friends.list[f].active;
@@ -1269,7 +1246,7 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
             pthread_mutex_unlock(&Winthread.lock);
 
             if (connection_status != TOX_CONNECTION_NONE) {
-                int colour = MAGENTA;
+                int colour;
 
                 switch (status) {
                     case TOX_USER_STATUS_NONE:
@@ -1283,6 +1260,10 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
                     case TOX_USER_STATUS_BUSY:
                         colour = RED;
                         break;
+
+                    default:
+                        colour = BAR_TEXT;
+                        break;
                 }
 
                 wattron(self->window, COLOR_PAIR(colour) | A_BOLD);
@@ -1314,7 +1295,7 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
 
                     statusmsg[s_len] = '\0';
 
-                    filter_str(statusmsg, s_len);
+                    filter_string(statusmsg, s_len, false);
 
                     pthread_mutex_lock(&Winthread.lock);
                     snprintf(Friends.list[f].statusmsg, sizeof(Friends.list[f].statusmsg), "%s", statusmsg);
@@ -1402,9 +1383,7 @@ static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
         wprintw(self->window, "Public key: ");
         wattroff(self->window, A_BOLD);
 
-        int i;
-
-        for (i = 0; i < TOX_PUBLIC_KEY_SIZE; ++i) {
+        for (int i = 0; i < TOX_PUBLIC_KEY_SIZE; ++i) {
             wprintw(self->window, "%02X", Friends.list[selected_num].pub_key[i] & 0xff);
         }
     }
@@ -1430,19 +1409,19 @@ void friendlist_onInit(ToxWindow *self, Toxic *toxic)
     getmaxyx(self->window, y2, x2);
 
     if (y2 <= 0 || x2 <= 0) {
-        exit_toxic_err("failed in friendlist_onInit", FATALERR_CURSES);
+        exit_toxic_err(FATALERR_CURSES, "failed in friendlist_onInit");
     }
 
     self->window_bar = subwin(self->window, WINDOW_BAR_HEIGHT, x2, y2 - 2, 0);
 }
 
-void disable_chatwin(uint32_t f_num)
+void disable_friend_window(uint32_t f_num)
 {
     if (f_num >= Friends.max_idx) {
         return;
     }
 
-    Friends.list[f_num].chatwin = -1;
+    Friends.list[f_num].window_id = -1;
 }
 
 #ifdef AUDIO
@@ -1459,25 +1438,21 @@ static void friendlist_onAV(ToxWindow *self, Toxic *toxic, uint32_t friend_numbe
     }
 
     Tox *tox = toxic->tox;
-    const Client_Config *c_config = toxic->c_config;
-    ToxWindow *home_window = toxic->home_window;
 
-    if (Friends.list[friend_number].chatwin == -1) {
-        if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-            if (state != TOXAV_FRIEND_CALL_STATE_FINISHED) {
-                Friends.list[friend_number].chatwin = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
-                set_active_window_index(Friends.list[friend_number].chatwin);
-            }
-        } else {
-            char nick[TOX_MAX_NAME_LENGTH];
-            get_nick_truncate(tox, nick, Friends.list[friend_number].num);
-            line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Audio action from: %s!", nick);
+    if (Friends.list[friend_number].window_id >= 0) {
+        return;
+    }
 
-            const char *errmsg = "* Warning: Too many windows are open.";
-            line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED, errmsg);
+    if (state != TOXAV_FRIEND_CALL_STATE_FINISHED) {
+        const int window_id = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
 
-            sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
+        if (window_id < 0) {
+            fprintf(stderr, "Failed to create new chat window in friendlist_onAV");
+            return;
         }
+
+        Friends.list[friend_number].window_id = window_id;
+        set_active_window_by_id(toxic->windows, window_id);
     }
 }
 #endif /* AUDIO */
@@ -1582,11 +1557,50 @@ bool friend_get_auto_accept_files(uint32_t friendnumber)
     return friend->auto_accept_files;
 }
 
+bool get_friend_public_key(char *pk, uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    memcpy(pk, friend->pub_key, TOX_PUBLIC_KEY_SIZE);
+    return true;
+}
+
+uint16_t get_friend_name(char *buf, size_t buf_size, uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        goto on_error;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        goto on_error;
+    }
+
+    snprintf(buf, buf_size, "%s", friend->name);
+    return (uint16_t) strlen(buf);
+
+on_error:
+    snprintf(buf, buf_size, "%s", UNKNOWN_NAME);
+    return (uint16_t) strlen(buf);
+}
+
 /*
  * Returns a pointer to the Friend_Settings object associated with `public_key`.
+ * If non-null, the friendnumber of `public_key` will be copied to the `friendnumber`
+ * pointer on success.
+ *
  * Returns NULL on failure.
  */
-static Friend_Settings *get_friend_settings_by_key(const char *public_key)
+static Friend_Settings *get_friend_settings_by_key(const char *public_key, uint32_t *friendnumber)
 {
     char pk_bin[TOX_PUBLIC_KEY_SIZE];
 
@@ -1602,6 +1616,10 @@ static Friend_Settings *get_friend_settings_by_key(const char *public_key)
         }
 
         if (memcmp(pk_bin, friend->pub_key, sizeof(friend->pub_key)) == 0) {
+            if (friendnumber != NULL) {
+                *friendnumber = friend->num;
+            }
+
             return &friend->settings;
         }
     }
@@ -1611,7 +1629,7 @@ static Friend_Settings *get_friend_settings_by_key(const char *public_key)
 
 bool friend_config_set_show_connection_msg(const char *public_key, bool show_connection_msg)
 {
-    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+    Friend_Settings *settings = get_friend_settings_by_key(public_key, NULL);
 
     if (settings == NULL) {
         return false;
@@ -1640,7 +1658,7 @@ bool friend_config_get_show_connection_msg(uint32_t friendnumber)
 
 bool friend_config_set_tab_name_colour(const char *public_key, const char *colour)
 {
-    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+    Friend_Settings *settings = get_friend_settings_by_key(public_key, NULL);
 
     if (settings == NULL) {
         return false;
@@ -1675,7 +1693,7 @@ int friend_config_get_tab_name_colour(uint32_t friendnumber)
 
 bool friend_config_set_autolog(const char *public_key, bool autolog_enabled)
 {
-    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+    Friend_Settings *settings = get_friend_settings_by_key(public_key, NULL);
 
     if (settings == NULL) {
         return false;
@@ -1704,7 +1722,7 @@ bool friend_config_get_autolog(uint32_t friendnumber)
 
 bool friend_config_set_auto_accept_files(const char *public_key, bool auto_accept_files)
 {
-    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+    Friend_Settings *settings = get_friend_settings_by_key(public_key, NULL);
 
     if (settings == NULL) {
         return false;
@@ -1731,12 +1749,69 @@ bool friend_config_get_auto_accept_files(uint32_t friendnumber)
     return friend->settings.auto_accept_files;
 }
 
+bool friend_config_alias_is_set(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        fprintf(stderr, "failed to get alias setting (invalid friendnumber %u)\n", friendnumber);
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->settings.alias_set;
+}
+
+bool friend_config_set_alias(const char *public_key, const char *alias, uint16_t length)
+{
+    uint32_t friendnumber;
+    Friend_Settings *settings = get_friend_settings_by_key(public_key, &friendnumber);
+
+    if (settings == NULL) {
+        return false;
+    }
+
+    if (friendnumber >= Friends.max_idx) {
+        return false;
+    }
+
+    if (alias == NULL || string_is_empty(alias)) {
+        fprintf(stderr, "Failed to set alias with NULL name for: %s\n", public_key);
+        return false;
+    }
+
+    if (length == 0 || length > TOXIC_MAX_NAME_LENGTH) {
+        fprintf(stderr, "Failed to set alias '%s' (invalid length: %u)\n", alias, length);
+        return false;
+    }
+
+    char tmp[TOXIC_MAX_NAME_LENGTH + 1];
+    const uint16_t tmp_len = copy_tox_str(tmp, sizeof(tmp), alias, length);
+    filter_string(tmp, tmp_len, true);
+
+    if (tmp_len == 0 || tmp_len > TOXIC_MAX_NAME_LENGTH) {
+        return false;
+    }
+
+    ToxicFriend *friend = &Friends.list[friendnumber];
+
+    snprintf(friend->name, sizeof(friend->name), "%s", tmp);
+    friend->namelength = strlen(friend->name);
+
+    settings->alias_set = true;
+
+    return true;
+}
+
 ToxWindow *new_friendlist(void)
 {
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
 
     if (ret == NULL) {
-        exit_toxic_err("failed in new_friendlist", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_friendlist");
     }
 
     ret->type = WINDOW_TYPE_FRIEND_LIST;
@@ -1744,6 +1819,7 @@ ToxWindow *new_friendlist(void)
     ret->onInit = &friendlist_onInit;
     ret->onKey = &friendlist_onKey;
     ret->onDraw = &friendlist_onDraw;
+    ret->onNickRefresh = &friendlist_onNickRefresh;
     ret->onFriendAdded = &friendlist_onFriendAdded;
     ret->onMessage = &friendlist_onMessage;
     ret->onConnectionChange = &friendlist_onConnectionChange;
@@ -1777,7 +1853,7 @@ ToxWindow *new_friendlist(void)
     Help *help = calloc(1, sizeof(Help));
 
     if (help == NULL) {
-        exit_toxic_err("failed in new_friendlist", FATALERR_MEMORY);
+        exit_toxic_err(FATALERR_MEMORY, "failed in new_friendlist");
     }
 
     ret->help = help;
