@@ -1,23 +1,9 @@
 /*  friendlist.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef FRIENDLIST_H
@@ -75,6 +61,7 @@ typedef struct Friend_Settings {
     bool  autolog;
     bool  auto_accept_files;
     bool  show_connection_msg;
+    bool  alias_set;
 } Friend_Settings;
 
 typedef struct {
@@ -84,7 +71,7 @@ typedef struct {
     size_t statusmsg_len;
     char pub_key[TOX_PUBLIC_KEY_SIZE];
     uint32_t num;
-    int chatwin;
+    int window_id;
     bool active;
     Tox_Connection connection_status;
     bool is_typing;
@@ -130,9 +117,9 @@ extern FriendsList Friends;
 
 ToxWindow *new_friendlist(void);
 void friendlist_onInit(ToxWindow *self, Toxic *toxic);
-void disable_chatwin(uint32_t f_num);
+void disable_friend_window(uint32_t f_num);
 int get_friendnum(uint8_t *name);
-void kill_friendlist(ToxWindow *self, const Client_Config *c_config);
+void kill_friendlist(ToxWindow *self, Windows *windows, const Client_Config *c_config);
 void friendlist_onFriendAdded(ToxWindow *self, Toxic *toxic, uint32_t num, bool sort);
 Tox_User_Status get_friend_status(uint32_t friendnumber);
 Tox_Connection get_friend_connection_status(uint32_t friendnumber);
@@ -166,6 +153,23 @@ void friend_set_auto_file_accept(uint32_t friendnumber, bool auto_accept);
  */
 bool friend_get_auto_accept_files(uint32_t friendnumber);
 
+/*
+ * Puts a NUL-terminated string representing a friend's name in `buf`. If
+ * `friendnumber` does not designate a valid friend number, a place-holder name
+ * is put in `buf`.
+ *
+ * Returns the length of the name.
+ */
+uint16_t get_friend_name(char *buf, size_t buf_size, uint32_t friendnumber);
+
+/*
+ * Puts a friend's public key in `pk`, which must have room for at least
+ * TOX_PUBLIC_KEY_SIZE bytes.
+ *
+ * Returns true on success.
+ */
+bool get_friend_public_key(char *pk, uint32_t friendnumber);
+
 /*
  * Enable or disable logging for this friend.
  */
@@ -233,4 +237,17 @@ bool friend_config_set_auto_accept_files(const char *public_key, bool autoaccept
  */
 bool friend_config_get_auto_accept_files(uint32_t friendnumber);
 
+/*
+ * Sets the friend's alias.
+ *
+ * Return true on success.
+ */
+bool friend_config_set_alias(const char *public_key, const char *alias, uint16_t length);
+
+/* Return true if the friend has an alias. */
+bool friend_config_alias_is_set(uint32_t friendnumber);
+
+/* Sets config settings to global defaults for all friends, ignoring friend-specific settings. */
+void friend_reset_default_config_settings(const Client_Config *c_config);
+
 #endif /* end of include guard: FRIENDLIST_H */
