@@ -1,23 +1,9 @@
 /*  global_commands.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include "global_commands.h"
@@ -150,7 +136,7 @@ void cmd_add_helper(ToxWindow *self, Toxic *toxic, const char *id_bin, const cha
             break;
     }
 
-    line_info_add(self, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+    line_info_add(self, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", errmsg);
 }
 
 void cmd_add(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
@@ -179,11 +165,12 @@ void cmd_add(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*arg
     if (space_idx > 0 && space_idx < arg_length - 1) {
         snprintf(msg, sizeof(msg), "%s", &id[space_idx + 1]);
     } else {
-        char selfname[TOX_MAX_NAME_LENGTH];
+        char selfname[TOX_MAX_NAME_LENGTH + 1];
         tox_self_get_name(tox, (uint8_t *) selfname);
 
-        size_t n_len = tox_self_get_name_size(tox);
+        const size_t n_len = tox_self_get_name_size(tox);
         selfname[n_len] = '\0';
+
         snprintf(msg, sizeof(msg), "Hello, my name is %s. Care to Tox?", selfname);
     }
 
@@ -253,7 +240,11 @@ void cmd_avatar(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
 
     path[len] = '\0';
     char filename[MAX_STR_SIZE];
-    get_file_name(filename, sizeof(filename), path);
+
+    if (get_file_name(filename, sizeof(filename), path) < 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to fetch file name (OOM)");
+        return;
+    }
 
     if (avatar_set(tox, path, len) == -1) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
@@ -432,11 +423,6 @@ void cmd_game(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
         return;
     }
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     const unsigned int id = rand_not_secure();
     const int ret = game_initialize(self, toxic, type, id, NULL, 0, true);
 
@@ -482,11 +468,6 @@ void cmd_conference(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, cha
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     if (argc < 1) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Please specify conference type: text | audio");
         return;
@@ -532,7 +513,7 @@ void cmd_conference(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, cha
 #endif
     }
 
-    if (init_conference_win(toxic, conferencenum, type, NULL, 0) == -1) {
+    if (init_conference_win(toxic, conferencenum, type, NULL, 0) < 0) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Conference window failed to initialize.");
         tox_conference_delete(tox, conferencenum, NULL);
         return;
@@ -562,11 +543,6 @@ void cmd_groupchat(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     if (argc < 1) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Group name required");
         return;
@@ -584,7 +560,7 @@ void cmd_groupchat(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
 
     snprintf(name, sizeof(name), "%s", argv[1]);
 
-    size_t nick_length = tox_self_get_name_size(tox);
+    const size_t nick_length = tox_self_get_name_size(tox);
     char self_nick[TOX_MAX_NAME_LENGTH + 1];
     tox_self_get_name(tox, (uint8_t *) self_nick);
     self_nick[nick_length] = '\0';
@@ -637,11 +613,6 @@ void cmd_join(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    if (get_num_active_windows() >= MAX_WINDOWS_NUM) {
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Warning: Too many windows are open.");
-        return;
-    }
-
     if (argc < 1) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Chat ID is required.");
         return;
@@ -719,40 +690,45 @@ void cmd_log(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*arg
 
     const Client_Config *c_config = toxic->c_config;
 
-    const char *msg;
     struct chatlog *log = self->chatwin->log;
 
     if (argc == 0) {
         if (log->log_on) {
-            msg = "Logging for this window is ON; type \"/log off\" to disable. (Logs are not encrypted)";
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                          "Logging to: %s\nType \"/log off\" to disable. Logs are not encrypted.", log->path);
         } else {
-            msg = "Logging for this window is OFF; type \"/log on\" to enable.";
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                          "Logging for this window is OFF; type \"/log on\" to enable.");
         }
 
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
         return;
     }
 
     const char *swch = argv[1];
 
     if (!strcmp(swch, "1") || !strcmp(swch, "on")) {
-        msg = log_enable(log) == 0 ? "Logging enabled." : "Warning: Failed to enable log.";
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+        if (log_enable(log) == 0) {
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Logging enabled to: %s", log->path);
+            return;
+        }
+
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to enable log.");
         return;
-    } else if (!strcmp(swch, "0") || !strcmp(swch, "off")) {
+    }
+
+    if (!strcmp(swch, "0") || !strcmp(swch, "off")) {
         if (self->type == WINDOW_TYPE_CHAT) {
             friend_set_logging_enabled(self->num, false);
         }
 
         log_disable(log);
 
-        msg = "Logging disabled.";
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Logging disabled.");
         return;
     }
 
-    msg = "Invalid option. Use \"/log on\" and \"/log off\" to toggle logging.";
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, msg);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                  "Invalid option. Use \"/log on\" and \"/log off\" to toggle logging.");
 }
 
 void cmd_myid(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
@@ -800,9 +776,10 @@ void cmd_myqr(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
         return;
     }
 
-    char nick[TOX_MAX_NAME_LENGTH];
+    char nick[TOX_MAX_NAME_LENGTH + 1];
     tox_self_get_name(tox, (uint8_t *) nick);
-    size_t nick_len = tox_self_get_name_size(tox);
+
+    const size_t nick_len = tox_self_get_name_size(tox);
     nick[nick_len] = '\0';
 
     const size_t data_file_len = strlen(toxic->client_data.data_path);
@@ -957,10 +934,11 @@ void cmd_nospam(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
     tox_self_set_nospam(tox, (uint32_t) nospam);
 
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Your new Tox ID is:");
+
     cmd_myid(window, self, toxic, 0, NULL);
-    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
+
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
-                  "Any services that relied on your old ID will need to be updated manually.");
+                  "\nAny services that relied on your old ID will need to be updated manually.");
     line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                   "If you ever want your old Tox ID back, type '/nospam %X'",
                   old_nospam);
@@ -1028,7 +1006,7 @@ void cmd_requests(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", FrndRequests.request[i].msg);
 
         if (++count < FrndRequests.num_requests) {
-            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, " ");
         }
     }
 }
@@ -1044,13 +1022,11 @@ void cmd_status(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
     Tox *tox = toxic->tox;
     const Client_Config *c_config = toxic->c_config;
 
-    const char *errmsg = NULL;
-
     lock_status();
 
     if (argc < 1) {
-        errmsg = "Require a status. Statuses are: online, busy and away.";
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                      "Require a status. Statuses are: online, busy and away.");
         goto finish;
     }
 
@@ -1064,8 +1040,8 @@ void cmd_status(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
     } else if (!strcasecmp(status_str, "busy")) {
         status = TOX_USER_STATUS_BUSY;
     } else {
-        errmsg = "Invalid status. Valid statuses are: online, busy and away.";
-        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                      "Invalid status. Valid statuses are: online, busy and away.");
         goto finish;
     }
 
