@@ -1,23 +1,9 @@
 /*  groupchat_commands.c
  *
+ *  Copyright (C) 2020-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include "groupchat_commands.h"
@@ -47,7 +33,7 @@ void cmd_chatid(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*
     char id_string[TOX_GROUP_CHAT_ID_SIZE * 2 + 1] = {0};
     char chat_id[TOX_GROUP_CHAT_ID_SIZE];
 
-    Tox_Err_Group_State_Queries err;
+    Tox_Err_Group_State_Query err;
 
     if (!tox_group_get_chat_id(toxic->tox, self->num, (uint8_t *) chat_id, &err)) {
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve the Chat ID (error %d).", err);
@@ -206,11 +192,11 @@ void cmd_kick(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
         return;
     }
 
-    Tox_Err_Group_Mod_Kick_Peer err;
-    tox_group_mod_kick_peer(tox, self->num, target_peer_id, &err);
+    Tox_Err_Group_Kick_Peer err;
+    tox_group_kick_peer(tox, self->num, target_peer_id, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_MOD_KICK_PEER_OK: {
+        case TOX_ERR_GROUP_KICK_PEER_OK: {
             char self_nick[TOX_MAX_NAME_LENGTH + 1];
             get_group_self_nick_truncate(tox, self_nick, self->num);
 
@@ -221,17 +207,17 @@ void cmd_kick(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*ar
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_KICK_PEER_PERMISSIONS: {
+        case TOX_ERR_GROUP_KICK_PEER_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,  "You do not have permission to kick %s.", nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_KICK_PEER_SELF: {
+        case TOX_ERR_GROUP_KICK_PEER_SELF: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,  "You cannot kick yourself.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_KICK_PEER_PEER_NOT_FOUND: {
+        case TOX_ERR_GROUP_KICK_PEER_PEER_NOT_FOUND: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,  "Specified nick or public key is invalid.");
             return;
         }
@@ -311,32 +297,32 @@ void cmd_mod(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*arg
         return;
     }
 
-    Tox_Err_Group_Mod_Set_Role err;
-    tox_group_mod_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_MODERATOR, &err);
+    Tox_Err_Group_Set_Role err;
+    tox_group_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_MODERATOR, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_MOD_SET_ROLE_OK: {
+        case TOX_ERR_GROUP_SET_ROLE_OK: {
             groupchat_onGroupModeration(self, toxic, self->num, self_peer_id, target_peer_id, TOX_GROUP_MOD_EVENT_MODERATOR);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_ROLE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to promote moderators.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_ASSIGNMENT: {
+        case TOX_ERR_GROUP_SET_ROLE_ASSIGNMENT: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,  "%s is already a moderator.", nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_SELF: {
+        case TOX_ERR_GROUP_SET_ROLE_SELF: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You cannot make yourself a moderator.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PEER_NOT_FOUND: {
+        case TOX_ERR_GROUP_SET_ROLE_PEER_NOT_FOUND: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "The specified nick or public key is invalid.");
             return;
         }
@@ -385,27 +371,27 @@ void cmd_unmod(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
         return;
     }
 
-    Tox_Err_Group_Mod_Set_Role err;
-    tox_group_mod_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_USER, &err);
+    Tox_Err_Group_Set_Role err;
+    tox_group_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_USER, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_MOD_SET_ROLE_OK: {
+        case TOX_ERR_GROUP_SET_ROLE_OK: {
             groupchat_onGroupModeration(self, toxic, self->num, self_peer_id, target_peer_id, TOX_GROUP_MOD_EVENT_USER);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_ROLE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,  "You do not have permission to unmod %s.",
                           nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_SELF: {
+        case TOX_ERR_GROUP_SET_ROLE_SELF: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You cannot remove your own moderator status.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PEER_NOT_FOUND: {
+        case TOX_ERR_GROUP_SET_ROLE_PEER_NOT_FOUND: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "The specified nick or public key is invalid.");
             return;
         }
@@ -437,11 +423,11 @@ void cmd_set_passwd(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, cha
         len = strlen(passwd);
     }
 
-    Tox_Err_Group_Founder_Set_Password err;
-    tox_group_founder_set_password(toxic->tox, self->num, (uint8_t *) passwd, len, &err);
+    Tox_Err_Group_Set_Password err;
+    tox_group_set_password(toxic->tox, self->num, (const uint8_t *) passwd, len, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_FOUNDER_SET_PASSWORD_OK: {
+        case TOX_ERR_GROUP_SET_PASSWORD_OK: {
             if (len > 0) {
                 line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Password has been set to %s.", passwd);
             } else {
@@ -451,13 +437,13 @@ void cmd_set_passwd(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, cha
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_PASSWORD_TOO_LONG: {
+        case TOX_ERR_GROUP_SET_PASSWORD_TOO_LONG: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Password length must not exceed %d.",
                           TOX_GROUP_MAX_PASSWORD_SIZE);
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_PASSWORD_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_PASSWORD_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to set the password.");
             return;
@@ -484,10 +470,10 @@ void cmd_set_peerlimit(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc,
     int maxpeers = 0;
 
     if (argc < 1) {
-        Tox_Err_Group_State_Queries err;
-        uint32_t maxpeers = tox_group_get_peer_limit(tox, self->num, &err);
+        Tox_Err_Group_State_Query err;
+        maxpeers = tox_group_get_peer_limit(tox, self->num, &err);
 
-        if (err != TOX_ERR_GROUP_STATE_QUERIES_OK) {
+        if (err != TOX_ERR_GROUP_STATE_QUERY_OK) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve peer limit (error %d).",
                           err);
             return;
@@ -504,16 +490,16 @@ void cmd_set_peerlimit(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc,
         return;
     }
 
-    Tox_Err_Group_Founder_Set_Peer_Limit err;
-    tox_group_founder_set_peer_limit(tox, self->num, maxpeers, &err);
+    Tox_Err_Group_Set_Peer_Limit err;
+    tox_group_set_peer_limit(tox, self->num, maxpeers, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_FOUNDER_SET_PEER_LIMIT_OK: {
+        case TOX_ERR_GROUP_SET_PEER_LIMIT_OK: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Peer limit has been set to %d.", maxpeers);
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_PEER_LIMIT_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_PEER_LIMIT_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to set the peer limit.");
             return;
@@ -540,10 +526,10 @@ void cmd_set_voice(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
     Tox_Group_Voice_State voice_state;
 
     if (argc < 1) {
-        Tox_Err_Group_State_Queries err;
+        Tox_Err_Group_State_Query err;
         voice_state = tox_group_get_voice_state(tox, self->num, &err);
 
-        if (err != TOX_ERR_GROUP_STATE_QUERIES_OK) {
+        if (err != TOX_ERR_GROUP_STATE_QUERY_OK) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve voice state (error %d).",
                           err);
             return;
@@ -587,16 +573,16 @@ void cmd_set_voice(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         return;
     }
 
-    Tox_Err_Group_Founder_Set_Voice_State err;
-    tox_group_founder_set_voice_state(tox, self->num, voice_state, &err);
+    Tox_Err_Group_Set_Voice_State err;
+    tox_group_set_voice_state(tox, self->num, voice_state, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_FOUNDER_SET_VOICE_STATE_OK: {
+        case TOX_ERR_GROUP_SET_VOICE_STATE_OK: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Voice state has been set to %s.", vstate_str);
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_VOICE_STATE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_VOICE_STATE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to set the voice state.");
             return;
@@ -624,10 +610,10 @@ void cmd_set_privacy(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, ch
     Tox_Group_Privacy_State privacy_state;
 
     if (argc < 1) {
-        Tox_Err_Group_State_Queries err;
+        Tox_Err_Group_State_Query err;
         privacy_state = tox_group_get_privacy_state(tox, self->num, &err);
 
-        if (err != TOX_ERR_GROUP_STATE_QUERIES_OK) {
+        if (err != TOX_ERR_GROUP_STATE_QUERY_OK) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve privacy state (error %d).",
                           err);
             return;
@@ -649,16 +635,16 @@ void cmd_set_privacy(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, ch
     privacy_state = strcasecmp(pstate_str,
                                "private") == 0 ? TOX_GROUP_PRIVACY_STATE_PRIVATE : TOX_GROUP_PRIVACY_STATE_PUBLIC;
 
-    Tox_Err_Group_Founder_Set_Privacy_State err;
-    tox_group_founder_set_privacy_state(tox, self->num, privacy_state, &err);
+    Tox_Err_Group_Set_Privacy_State err;
+    tox_group_set_privacy_state(tox, self->num, privacy_state, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_FOUNDER_SET_PRIVACY_STATE_OK: {
+        case TOX_ERR_GROUP_SET_PRIVACY_STATE_OK: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Privacy state has been set to %s.", pstate_str);
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_PRIVACY_STATE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_PRIVACY_STATE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to set the privacy state.");
             return;
@@ -686,10 +672,10 @@ void cmd_set_topic_lock(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc,
     const char *tlock_str = NULL;
 
     if (argc < 1) {
-        Tox_Err_Group_State_Queries err;
+        Tox_Err_Group_State_Query err;
         topic_lock = tox_group_get_topic_lock(tox, self->num, &err);
 
-        if (err != TOX_ERR_GROUP_STATE_QUERIES_OK) {
+        if (err != TOX_ERR_GROUP_STATE_QUERY_OK) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve topic lock (error %d).",
                           err);
             return;
@@ -710,16 +696,16 @@ void cmd_set_topic_lock(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc,
     topic_lock = strcasecmp(tlock_str, "on") == 0 ? TOX_GROUP_TOPIC_LOCK_ENABLED : TOX_GROUP_TOPIC_LOCK_DISABLED;
     const char *display_str = (topic_lock ==  TOX_GROUP_TOPIC_LOCK_ENABLED) ? "enabled" : "disabled";
 
-    Tox_Err_Group_Founder_Set_Topic_Lock err;
-    tox_group_founder_set_topic_lock(tox, self->num, topic_lock, &err);
+    Tox_Err_Group_Set_Topic_Lock err;
+    tox_group_set_topic_lock(tox, self->num, topic_lock, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_FOUNDER_SET_TOPIC_LOCK_OK: {
+        case TOX_ERR_GROUP_SET_TOPIC_LOCK_OK: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Topic lock has been %s.", display_str);
             return;
         }
 
-        case TOX_ERR_GROUP_FOUNDER_SET_TOPIC_LOCK_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_TOPIC_LOCK_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                           "You do not have permission to set the topic lock.");
             return;
@@ -763,32 +749,32 @@ void cmd_silence(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (
         return;
     }
 
-    Tox_Err_Group_Mod_Set_Role err;
-    tox_group_mod_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_OBSERVER, &err);
+    Tox_Err_Group_Set_Role err;
+    tox_group_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_OBSERVER, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_MOD_SET_ROLE_OK: {
+        case TOX_ERR_GROUP_SET_ROLE_OK: {
             groupchat_onGroupModeration(self, toxic, self->num, self_peer_id, target_peer_id, TOX_GROUP_MOD_EVENT_OBSERVER);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_ROLE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You do not have permission to silence %s.",
                           nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_ASSIGNMENT: {
+        case TOX_ERR_GROUP_SET_ROLE_ASSIGNMENT: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s is already silenced.", nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_SELF: {
+        case TOX_ERR_GROUP_SET_ROLE_SELF: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You cannot silence yourself.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PEER_NOT_FOUND: {
+        case TOX_ERR_GROUP_SET_ROLE_PEER_NOT_FOUND: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "The specified nick or public key is invalid.");
             return;
         }
@@ -836,32 +822,32 @@ void cmd_unsilence(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         return;
     }
 
-    Tox_Err_Group_Mod_Set_Role err;
-    tox_group_mod_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_USER, &err);
+    Tox_Err_Group_Set_Role err;
+    tox_group_set_role(tox, self->num, target_peer_id, TOX_GROUP_ROLE_USER, &err);
 
     switch (err) {
-        case TOX_ERR_GROUP_MOD_SET_ROLE_OK: {
+        case TOX_ERR_GROUP_SET_ROLE_OK: {
             groupchat_onGroupModeration(self, toxic, self->num, self_peer_id, target_peer_id, TOX_GROUP_MOD_EVENT_USER);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PERMISSIONS: {
+        case TOX_ERR_GROUP_SET_ROLE_PERMISSIONS: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You do not have permission to unsilence %s.",
                           nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_ASSIGNMENT: {
+        case TOX_ERR_GROUP_SET_ROLE_ASSIGNMENT: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s is not silenced.", nick);
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_SELF: {
+        case TOX_ERR_GROUP_SET_ROLE_SELF: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "You cannot unsilence yourself.");
             return;
         }
 
-        case TOX_ERR_GROUP_MOD_SET_ROLE_PEER_NOT_FOUND: {
+        case TOX_ERR_GROUP_SET_ROLE_PEER_NOT_FOUND: {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "The specified nick or public key is invalid.");
             return;
         }
@@ -907,10 +893,10 @@ void cmd_set_topic(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
     const Client_Config *c_config = toxic->c_config;
 
     if (argc < 1) {
-        Tox_Err_Group_State_Queries err;
+        Tox_Err_Group_State_Query err;
         size_t tlen = tox_group_get_topic_size(tox, self->num, &err);
 
-        if (err != TOX_ERR_GROUP_STATE_QUERIES_OK) {
+        if (err != TOX_ERR_GROUP_STATE_QUERY_OK) {
             line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to retrieve topic length (error %d).",
                           err);
             return;
@@ -936,7 +922,7 @@ void cmd_set_topic(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
     const char *topic = argv[1];
 
     Tox_Err_Group_Topic_Set err;
-    tox_group_set_topic(tox, self->num, (uint8_t *) topic, strlen(topic), &err);
+    tox_group_set_topic(tox, self->num, (const uint8_t *) topic, strlen(topic), &err);
 
     switch (err) {
         case TOX_ERR_GROUP_TOPIC_SET_OK: {
@@ -966,14 +952,16 @@ void cmd_set_topic(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char
         }
     }
 
+    groupchat_update_statusbar_topic(self, tox);
+
     char self_nick[TOX_MAX_NAME_LENGTH + 1];
     get_group_self_nick_truncate(tox, self_nick, self->num);
 
-    line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, 1, MAGENTA, "-!- You set the topic to: %s", topic);
-
     char tmp_event[MAX_STR_SIZE];
-    snprintf(tmp_event, sizeof(tmp_event), "set topic to %s", topic);
-    write_to_log(self->chatwin->log, c_config, tmp_event, self_nick, true);
+    snprintf(tmp_event, sizeof(tmp_event), "-!- You set the topic to: %s", topic);
+
+    line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, 1, MAGENTA, "%s", tmp_event);
+    write_to_log(self->chatwin->log, c_config, tmp_event, NULL, LOG_HINT_TOPIC);
 }
 
 void cmd_unignore(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
@@ -1073,7 +1061,7 @@ void cmd_whois(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
             peer_id = peer->peer_id;
         }
 
-        int peer_index = get_peer_index(self->num, peer_id);
+        const int peer_index = get_peer_index(self->num, peer_id);
 
         if (peer_index < 0) {
             continue;
@@ -1099,15 +1087,22 @@ void cmd_whois(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
             role_str = "Observer";
         }
 
+
         char last_seen_str[128];
-        get_elapsed_time_str_alt(last_seen_str, sizeof(last_seen_str),
-                                 get_unix_time() - peer->last_active);
+        const bool is_self = peer_id == tox_group_self_get_peer_id(tox, self->num, NULL);
+
+        if (is_self) {
+            last_seen_str[0] = '-';
+            last_seen_str[1] = '\0';
+        } else {
+            get_elapsed_time_str_alt(last_seen_str, sizeof(last_seen_str), get_unix_time() - peer->last_active);
+        }
 
         char pk_string[TOX_GROUP_PEER_PUBLIC_KEY_SIZE * 2 + 1] = {0};
 
-        for (size_t i = 0; i < TOX_GROUP_PEER_PUBLIC_KEY_SIZE; ++i) {
+        for (size_t j = 0; j < TOX_GROUP_PEER_PUBLIC_KEY_SIZE; ++j) {
             char d[3];
-            snprintf(d, sizeof(d), "%02X", peer->public_key[i] & 0xff);
+            snprintf(d, sizeof(d), "%02X", peer->public_key[j] & 0xff);
             strcat(pk_string, d);
         }
 
@@ -1149,4 +1144,3 @@ void cmd_whois(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
         line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Last active: %s", last_seen_str);
     }
 }
-
