@@ -1,23 +1,9 @@
 /*  groupchats.h
  *
+ *  Copyright (C) 2020-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef GROUPCHATS_H
@@ -30,7 +16,7 @@
 #define SIDEBAR_WIDTH 16
 #endif
 
-#define MAX_GROUPCHAT_NUM (MAX_WINDOWS_NUM - 2)
+#define MAX_GROUPCHAT_NUM 100
 
 typedef enum Group_Join_Type {
     Group_Join_Type_Create,
@@ -67,7 +53,7 @@ typedef struct {
     bool       active;
     uint64_t   time_connected;    /* The time we successfully connected to the group */
 
-    int        chatwin;
+    uint16_t   window_id;
     int        side_pos;     /* current position of the sidebar - used for scrolling up and down */
 } GroupChat;
 
@@ -85,6 +71,9 @@ void groupchat_onGroupModeration(ToxWindow *self, Toxic *toxic, uint32_t groupnu
 
 void groupchat_rejoin(ToxWindow *self, Toxic *toxic);
 
+/* Updates the groupchat topic in the top statusbar. */
+void groupchat_update_statusbar_topic(ToxWindow *self, const Tox *tox);
+
 /* Puts the peer_id associated with `identifier` in `peer_id`. The string may be
  * either a nick or a public key.
  *
@@ -127,7 +116,7 @@ void group_toggle_peer_ignore(uint32_t groupnumber, int peer_id, bool ignore);
  *
  * Return true on success.
  */
-bool groupchat_config_set_tab_name_colour(const char *public_key, const char *colour);
+bool groupchat_config_set_tab_name_colour(Windows *windows, const char *public_key, const char *colour);
 
 /*
  * Sets the auto-logging preference for the groupchat associated with `public_key`.
@@ -136,6 +125,6 @@ bool groupchat_config_set_tab_name_colour(const char *public_key, const char *co
  *
  * Return true on success.
  */
-bool groupchat_config_set_autolog(const char *public_key, bool autolog_enabled);
+bool groupchat_config_set_autolog(Windows *windows, const char *public_key, bool autolog_enabled);
 
 #endif /* #define GROUPCHATS_H */
