@@ -1,23 +1,9 @@
 /*  misc_tools.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 #ifndef MISC_TOOLS_H
 #define MISC_TOOLS_H
@@ -28,6 +14,10 @@
 #include "toxic.h"
 #include "windows.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #ifndef MIN
 #define MIN(x, y) (((x) < (y)) ? (x) : (y))
 #endif
@@ -102,11 +92,19 @@ time_t get_unix_time(void);
  */
 void get_time_str(char *buf, size_t bufsize, const char *format_string);
 
-/* Converts seconds to string in format H hours, m minutes, s seconds */
-void get_elapsed_time_str_alt(char *buf, int bufsize, uint64_t secs);
+/* Converts `elapsed_seconds` to string in format "H hours, m minutes, s seconds" and puts the resulting
+ * string in `buf`.
+ *
+ * If `elapsed_seconds` is zero, an error message is placed in `buf`.
+ */
+void get_elapsed_time_str_alt(char *buf, int bufsize, uint64_t elapsed_seconds);
 
-/* Converts seconds to string in format HH:mm:ss; truncates hours and minutes when necessary */
-void get_elapsed_time_str(char *buf, int bufsize, time_t secs);
+/* Converts `elapsed_seconds` to string in format "HH:mm:ss"; truncates hours and minutes when necessary.
+ * Puts the resulting string in `buf.
+ *
+ * If `elapsed_seconds` is zero, an error message is placed in `buf`.
+ */
+void get_elapsed_time_str(char *buf, int bufsize, uint64_t elapsed_seconds);
 
 /* get the current local time (not thread safe) */
 struct tm *get_time(void);
@@ -158,17 +156,29 @@ int qsort_ptr_char_array_helper(const void *str1, const void *str2);
  * A valid toxic nick:
  * - cannot be empty
  * - cannot start with a space
- * - must not contain a forward slash (for logfile naming purposes)
- * - must not contain contiguous spaces
- * - must not contain a newline or tab seqeunce
+ * - cannot contain contiguous spaces
+ * - cannot contain forward slash (for log file naming purposes)
+ * - cannot contain any characters in the `invalid_nick_chars` array that
+ *   resides in misc_tools.c.
  */
 bool valid_nick(const char *nick);
 
-/* Converts all newline/tab chars to spaces (use for strings that should be contained to a single line) */
-void filter_str(char *str, size_t len);
+/* Converts all invalid single-line string characters to spaces (newlines,
+ * tabs, nul etc.).
+ *
+ * If `is_nick` is true, additional characters specific to nicks are filtered.
+ *
+ * This function is used for strings that should be human-readable, and contained
+ * to a single line, such as status messages, nicks, and group topics.
+ */
+void filter_string(char *str, size_t len, bool is_nick);
 
-/* gets base file name from path or original file name if no path is supplied */
-size_t get_file_name(char *namebuf, size_t bufsize, const char *pathname);
+/* Gets base file name from path or original file name if no path is supplied.
+ *
+ * Returns the file name length on success.
+ * Returns -1 on failure (OOM).
+ */
+int get_file_name(char *namebuf, size_t bufsize, const char *pathname);
 
 /* Gets the base directory of path and puts it in dir.
  * dir must have at least as much space as path_len.
@@ -181,9 +191,17 @@ size_t get_base_dir(const char *path, size_t path_len, char *dir);
 /* converts str to all lowercase */
 void str_to_lower(char *str);
 
-/* puts friendnum's nick in buf, truncating at TOXIC_MAX_NAME_LENGTH if necessary.
-   Returns nick len on success, -1 on failure */
-size_t get_nick_truncate(Tox *tox, char *buf, uint32_t friendnum);
+/* Gets the name designated by `friendnum` via tox API call and puts it in buf,
+ * truncating the name at `buf_size`
+ *
+ * If the toxcore API call fails, put UNKNOWN_NAME in buf.
+ *
+ * Note: This function should only be used when initially adding a new friend. In all
+ * other instances, use the `get_friend_name()` function from friendlist.h.
+ *
+ * Returns the length of the resulting nick.
+ */
+size_t get_nick_truncate(Tox *tox, char *buf, uint16_t buf_size, uint32_t friendnum);
 
 /* same as get_nick_truncate but for conferences */
 int get_conference_nick_truncate(Tox *tox, char *buf, uint32_t peernum, uint32_t conferencenum);
@@ -194,8 +212,11 @@ size_t get_group_nick_truncate(Tox *tox, char *buf, uint32_t peer_id, uint32_t g
 /* same as get_group_nick_truncate() but for self. */
 size_t get_group_self_nick_truncate(Tox *tox, char *buf, uint32_t groupnum);
 
-/* copies data to msg buffer.
-   returns length of msg, which will be no larger than size-1 */
+/* Copies up to `size` bytes from the `data` string of length `length` to the `msg` buffer,
+ * replacing all \t and \v bytes with spaces, and removing all \r bytes.
+ *
+ * Returns the length of the resulting string in `msg`, which is guaranteed to be NUL-terminated.
+ */
 size_t copy_tox_str(char *msg, size_t size, const char *data, size_t length);
 
 /* returns index of the first instance of ch in s starting at idx.
@@ -266,4 +287,14 @@ unsigned int rand_not_secure(void);
  */
 int colour_string_to_int(const char *colour);
 
+/*
+ * A wrapper for strftime() from time.h. We use this wrapper to suppress
+ * gcc compiler warnings produced by the -Wformat-nonliteral flag.
+ */
+size_t format_time_str(char *s, size_t max, const char *format, const struct tm *tm);
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif /* __cplusplus */
+
 #endif /* MISC_TOOLS_H */
