@@ -1,23 +1,9 @@
 /*  toxic.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef TOXIC_H
@@ -39,6 +25,13 @@
 #define SIGINT 2
 #endif
 
+#include <assert.h>
+#if !defined(static_assert) && (defined(__GNUC__) || defined(__clang__)) \
+    && defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L \
+    && __STDC_VERSION__ <= 201710L
+#define static_assert _Static_assert
+#endif
+
 #include <stdbool.h>
 #include <curses.h>
 
@@ -51,6 +44,10 @@
 #include "settings.h"
 #include "toxic_constants.h"
 
+#ifdef X11
+#include "x11focus.h"
+#endif
+
 typedef struct Client_Data {
     bool is_encrypted;
     char pass[MAX_PASSWORD_LEN + 1];
@@ -63,18 +60,32 @@ typedef struct ToxAV ToxAV;
 typedef struct ToxWindow ToxWindow;
 typedef struct Run_Options Run_Options;
 
+typedef struct Windows {
+    ToxWindow  **list;
+    uint16_t   count;
+    uint16_t   active_index;
+} Windows;
+
 typedef struct Toxic {
     Tox   *tox;
 #ifdef AUDIO
     ToxAV *av;
 #endif
+
+#ifdef X11
+    X11_Focus     x11_focus;
+#endif
+
     Client_Data   client_data;
     Client_Config *c_config;
     Run_Options   *run_opts;
 
     ToxWindow     *home_window;
+    Windows       *windows;
 } Toxic;
 
+typedef struct Init_Queue Init_Queue;
+
 void lock_status(void);
 void unlock_status(void);
 
@@ -84,10 +95,12 @@ void flag_interface_refresh(void);
 void set_window_refresh_rate(size_t refresh_rate);
 
 void exit_toxic_success(Toxic *toxic) __attribute__((__noreturn__));
-void exit_toxic_err(const char *errmsg, int errcode) __attribute__((__noreturn__));
+void exit_toxic_err(int errcode, const char *errmsg, ...) __attribute__((__noreturn__, format(printf, 2, 3)));
 
 int store_data(const Toxic *toxic);
 
+void init_term(const Client_Config *c_config, Init_Queue *init_q, bool use_default_locale);
+
 /* callbacks */
 void on_friend_request(Tox *tox, const uint8_t *public_key, const uint8_t *data, size_t length, void *userdata);
 void on_friend_connection_status(Tox *tox, uint32_t friendnumber, Tox_Connection status, void *userdata);
@@ -121,10 +134,9 @@ void on_group_invite(Tox *tox, uint32_t friendnumber, const uint8_t *invite_data
                      const uint8_t *group_name,
                      size_t group_name_length, void *userdata);
 void on_group_message(Tox *tox, uint32_t groupnumber, uint32_t peernumber, TOX_MESSAGE_TYPE type,
-                      const uint8_t *message, size_t length, uint32_t message_id, void *userdata);
+                      const uint8_t *message, size_t length, Tox_Group_Message_Id message_id, void *userdata);
 void on_group_private_message(Tox *tox, uint32_t groupnumber, uint32_t peernumber, TOX_MESSAGE_TYPE type,
-                              const uint8_t *message, size_t length,
-                              void *userdata);
+                              const uint8_t *message, size_t length, Tox_Group_Message_Id message_id, void *userdata);
 void on_group_peer_join(Tox *tox, uint32_t groupnumber, uint32_t peernumber, void *userdata);
 void on_group_peer_exit(Tox *tox, uint32_t groupnumber, uint32_t peer_id, Tox_Group_Exit_Type exit_type,
                         const uint8_t *nick,
