@@ -1,23 +1,9 @@
 /*  video_call.c
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #include "chat_commands.h"
@@ -109,9 +95,16 @@ static void read_video_device_callback(Toxic *toxic, int16_t width, int16_t heig
     }
 
     const Client_Config *c_config = toxic->c_config;
+
     ToxWindow *home_window = toxic->home_window;
 
     uint32_t friend_number = *((uint32_t *)data); /* TODO: Or pass an array of call_idx's */
+
+    if (friend_number >= CallControl.max_calls) {
+        line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid call index.");
+        return;
+    }
+
     Call *this_call = &CallControl.calls[friend_number];
     Toxav_Err_Send_Frame error;
 
@@ -216,6 +209,10 @@ void on_video_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t video_bit_rat
     UNUSED_VAR(av);
     UNUSED_VAR(user_data);
 
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *call = &CallControl.calls[friend_number];
     call->video_bit_rate = video_bit_rate;
 
@@ -227,6 +224,10 @@ void on_video_bit_rate(ToxAV *av, uint32_t friend_number, uint32_t video_bit_rat
 
 void callback_recv_video_starting(uint32_t friend_number)
 {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *this_call = &CallControl.calls[friend_number];
 
     if (this_call->status != cs_Active || this_call->vout_idx != -1) {
@@ -237,6 +238,10 @@ void callback_recv_video_starting(uint32_t friend_number)
 }
 void callback_recv_video_end(uint32_t friend_number)
 {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *this_call = &CallControl.calls[friend_number];
 
     if (this_call->status != cs_Active || this_call->vout_idx == -1) {
@@ -248,25 +253,37 @@ void callback_recv_video_end(uint32_t friend_number)
 }
 static void callback_video_starting(Toxic *toxic, uint32_t friend_number)
 {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     Call *this_call = &CallControl.calls[friend_number];
 
     Toxav_Err_Call_Control error = TOXAV_ERR_CALL_CONTROL_OK;
     toxav_call_control(toxic->av, friend_number, TOXAV_CALL_CONTROL_SHOW_VIDEO, &error);
 
-    if (error == TOXAV_ERR_CALL_CONTROL_OK) {
-        for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-            ToxWindow *window = get_window_ptr(i);
+    Windows *windows = toxic->windows;
 
-            if (window != NULL && window->is_call && window->num == friend_number) {
-                if (start_video_transmission(window, toxic, this_call) == 0) {
-                    line_info_add(window, toxic->c_config, NULL, NULL, NULL, SYS_MSG, 0, 0, "Video capture starting.");
-                }
+    if (error == TOXAV_ERR_CALL_CONTROL_OK) {
+        for (uint16_t i = 0; i < windows->count; ++i) {
+            ToxWindow *window = windows->list[i];
+
+            if (window->is_call && window->num != friend_number) {
+                continue;
+            }
+
+            if (start_video_transmission(window, toxic, this_call) == 0) {
+                line_info_add(window, toxic->c_config, NULL, NULL, NULL, SYS_MSG, 0, 0, "Video capture starting.");
             }
         }
     }
 }
 void callback_video_end(uint32_t friend_number)
 {
+    if (friend_number >= CallControl.max_calls) {
+        return;
+    }
+
     stop_video_transmission(&CallControl.calls[friend_number], friend_number);
 }
 /*
@@ -304,6 +321,11 @@ void cmd_vcall(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
         return;
     }
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_None) {
@@ -329,6 +351,11 @@ void cmd_video(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*a
 
     const Client_Config *c_config = toxic->c_config;
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *this_call = &CallControl.calls[self->num];
 
     if (argc != 0) {
@@ -369,6 +396,11 @@ void cmd_res(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*arg
 
     const Client_Config *c_config = toxic->c_config;
 
+    if (self->num >= CallControl.max_calls) {
+        print_err(self, c_config, "Invalid call index.");
+        return;
+    }
+
     Call *call = &CallControl.calls[self->num];
 
     if (argc == 0) {
