@@ -1,23 +1,9 @@
 /*  windows.h
  *
+ *  Copyright (C) 2014-2024 Toxic All Rights Reserved.
  *
- *  Copyright (C) 2024 Toxic All Rights Reserved.
- *
- *  This file is part of Toxic.
- *
- *  Toxic is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  Toxic is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Toxic.  If not, see <http://www.gnu.org/licenses/>.
- *
+ *  This file is part of Toxic. Toxic is free software licensed
+ *  under the GNU General Public License 3.0.
  */
 
 #ifndef WINDOWS_H
@@ -37,7 +23,6 @@
 #include "settings.h"
 #include "toxic.h"
 
-#define MAX_WINDOWS_NUM 20
 #define MAX_WINDOW_NAME_LENGTH 22
 #define CURS_Y_OFFSET 1    /* y-axis cursor offset for chat contexts */
 #define CHATBOX_HEIGHT 1
@@ -157,12 +142,11 @@ typedef struct GameData GameData;
 #endif
 
 struct ToxWindow {
-    /* ncurses */
     bool(*onKey)(ToxWindow *, Toxic *, wint_t, bool);
     void(*onDraw)(ToxWindow *, Toxic *);
     void(*onInit)(ToxWindow *, Toxic *);
+    void(*onNickRefresh)(ToxWindow *, Toxic *);
 
-    /* toxcore */
     void(*onFriendRequest)(ToxWindow *, Toxic *, const char *, const char *, size_t);
     void(*onFriendAdded)(ToxWindow *, Toxic *, uint32_t, bool);
     void(*onConnectionChange)(ToxWindow *, Toxic *, uint32_t, Tox_Connection);
@@ -228,7 +212,7 @@ struct ToxWindow {
     char name[TOXIC_MAX_NAME_LENGTH + 1];
     int colour;  /* The ncurses colour pair of the window name */
     uint32_t num;    /* corresponds to friendnumber in chat windows */
-    uint8_t index; /* This window's index in the windows array */
+    uint16_t id; /* a unique and permanent identifier for this window */
     bool scroll_pause; /* true if this window is not scrolled to the bottom */
     unsigned int pending_messages;  /* # of new messages in this window since the last time it was focused */
     int x;
@@ -251,8 +235,6 @@ struct ToxWindow {
     WINDOW *window_bar;
 };
 
-extern ToxWindow *windows[MAX_WINDOWS_NUM];
-
 /* statusbar info holder */
 struct StatusBar {
     WINDOW *topline;
@@ -262,6 +244,8 @@ struct StatusBar {
     size_t nick_len;
     Tox_User_Status status;
     Tox_Connection connection;
+    char topic[TOX_GROUP_MAX_TOPIC_LENGTH + 1];
+    size_t topic_len;
 };
 
 #ifdef AUDIO
@@ -325,16 +309,31 @@ struct Help {
 
 void init_windows(Toxic *toxic);
 void draw_active_window(Toxic *toxic);
-int add_window(Toxic *toxic, ToxWindow *w);
-void del_window(ToxWindow *w, const Client_Config *c_config);
-void set_active_window_index(uint8_t index);
-int get_num_active_windows(void);
+void del_window(ToxWindow *w, Windows *windows, const Client_Config *c_config);
 void kill_all_windows(Toxic *toxic);    /* should only be called on shutdown */
-void on_window_resize(void);
+void on_window_resize(Windows *windows);
 void force_refresh(WINDOW *w);
-ToxWindow *get_window_ptr(size_t i);
-ToxWindow *get_active_window(void);
-void draw_window_bar(ToxWindow *self);
+ToxWindow *get_window_pointer_by_id(Windows *windows, uint16_t id);
+ToxWindow *get_active_window(const Windows *windows);
+void draw_window_bar(ToxWindow *self, Windows *windows);
+
+/*
+ * Initializes window `w` and adds it to the windows list.
+ *
+ * Returns the window's unique ID on success.
+ * Returns -1 on failure.
+ */
+int add_window(Toxic *toxic, ToxWindow *w);
+
+/*
+ * Sets the active window to the window associated with `id`.
+ */
+void set_active_window_by_id(Windows *windows, uint16_t id);
+
+/*
+ * Sets the active window to the first found window of window type `type`.
+ */
+void set_active_window_by_type(Windows *windows, Window_Type type);
 
 /*
  * Enables and disables the log associated the ToxWindow of type `type` and with
@@ -342,20 +341,26 @@ void draw_window_bar(ToxWindow *self);
  *
  * Returns true on success.
  */
-bool enable_window_log_by_number_type(uint32_t number, Window_Type type);
-bool disable_window_log_by_number_type(uint32_t number, Window_Type type);
+bool enable_window_log_by_number_type(Windows *windows, uint32_t number, Window_Type type);
+bool disable_window_log_by_number_type(Windows *windows, uint32_t number, Window_Type type);
 
 /*
  * Returns a pointer to the ToxWindow of type `type` and with num `number`.
  * Returns NULL if a window with those parameters doesn't exist.
  */
-ToxWindow *get_window_by_number_type(uint32_t number, Window_Type type);
+ToxWindow *get_window_by_number_type(Windows *windows, uint32_t number, Window_Type type);
 
 /* Returns the number of active windows of given type. */
-size_t get_num_active_windows_type(Window_Type type);
+uint16_t get_num_active_windows_type(const Windows *windows, Window_Type type);
 
 /* refresh inactive windows to prevent scrolling bugs.
    call at least once per second */
-void refresh_inactive_windows(const Client_Config *c_config);
+void refresh_inactive_windows(Windows *windows, const Client_Config *c_config);
+
+/*
+ * Updates the friend name associated with the given window. Called
+ * after config changes.
+ */
+void refresh_window_names(Toxic *toxic);
 
 #endif // WINDOWS_H
